# +========================================================================+
# || CipUX::Task                                                          ||
# ||                                                                      ||
# || Defines user tasks for console and GUI                               ||
# ||                                                                      ||
# || (c) Copyright, Urheberrecht 1999 - 2006 by Christian Kuelker         ||
# ||                                                                      ||
# || License: GPL version 2                                               ||
# ||                                                                      ||
# || Revision: 2006-04-05                                                 ||
# ||                                                                      ||
# +========================================================================+
package CipUX::Task;

use 5.006000;
use strict;
use warnings;

use vars qw(@future @farfuture %cipux);

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use CipUX::Task ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
#user - course
#uid  - gid (group)

our @EXPORT = qw(
		%task
		%option
		&taskusage
                &task_exec_cmd_writing

		&cipux_task_create_object

		&cipux_task_create_user
		&cipux_task_list_users
		&cipux_task_change_user_password
		&cipux_task_backup_user_homedir
		&cipux_task_delete_user_homedir
		&cipux_task_delete_user
		&cipux_task_disable_user
		&cipux_task_enable_user
		&cipux_task_restore_user_homedir
		&cipux_task_reset_user_homedir
		&cipux_task_add_user_to_course

		&cipux_task_create_course
		&cipux_task_change_course_password
		&cipux_task_delete_course
		&cipux_task_disable_course
                &cipux_task_enable_course
		&cipux_task_list_courses
		&cipux_task_list_user_members_of_course
		&cipux_task_remove_nonexistent_users_from_all_courses
		&cipux_task_remove_user_from_course

		&cipux_task_create_role
		&cipux_task_delete_role
		&cipux_task_list_roles

		&cipux_task_create_skel
		&cipux_task_delete_skel
		&cipux_task_list_skels


                &cipux_task_list_group_node_role
		&cipux_task_list_user_node_role
		&cipux_task_list_teachers
     		&cipux_task_list_students
		&cipux_task_list_guests
		&cipux_task_list_roles
		&cipux_task_list_objects
		&cipux_task_list_rpc_functions


		&cipux_task_change_cipux_config_variable_value

		&cipux_task_list_rooms
		&cipux_task_create_room
		&cipux_task_delete_room
		&cipux_task_list_machines
		&cipux_task_create_machine
		&cipux_task_delete_machine
		&cipux_task_list_hardware_types
		&cipux_task_create_hardware_type
		&cipux_task_delete_hardware_type
		&cipux_task_list_image_slots
		&cipux_task_create_image_slot
		&cipux_task_delete_image_slot
		&cipux_task_list_images
		&cipux_task_create_image
		&cipux_task_delete_image

		&cipux_task_create_applicant
		&cipux_task_print_apply_for_admission

		&cipux_task_disable_user_posix_account
		&cipux_task_disable_user_samba_account
		&cipux_task_disable_course_posix_account
		&cipux_task_disable_course_samba_account
		&cipux_task_enable_user_posix_account
		&cipux_task_enable_user_samba_account
		&cipux_task_enable_course_posix_account
		&cipux_task_enable_course_samba_account

		&cipux_task_create_examination_course
		&cipux_task_create_examination_user

		&cipux_task_tell_me_user_firstname
		&cipux_task_tell_me_user_lastname
                &cipux_task_tell_me_user_canonical_name 
		&cipux_task_tell_me_user_role

		&cipux_task_change_own_password
		&cipux_task_change_user_role
);
# add this feature in future
my @future = qw(

		&cipux_task_disable_course
		&cipux_task_enable_course
		&cipux_task_extent_user_exipredate
		&cipux_task_set_user_userquota
		&cipux_task_send_user_expires_warning_mail
		&cipux_task_send_user_quota_warning_mail
		&cipux_task_enable_user_accountless_printing
		&cipux_task_disable_user_acountless_printing
		&cipux_task_enable_user_accounting
		&cipux_task_enable_user_printing
		&cipux_task_disable_user_accounting
		&cipux_task_disable_user_printing
		&cipux_task_set_user_accounting_money_in_cent_for_printing
		&cipux_task_send_user_warning_mail_for_low_accounting_money
		&cipux_task_change_user_password_by_user
		&cipux_task_set_user_password_by_admin
);

# add this feature in FAR future
@farfuture =qw(
	       &cipux_task_set_user_groupquota
	       &cipux_task_add_user_to_group_as_admin
	       &cipux_task_add_user_to_group_as_jadmin
	       &cipux_task_remove_user_fom_group_as_admin
	       &cipux_task_remove_user_fom_group_jadmin
);

our $VERSION = '3.002010';

=head1 NAME

CipUX::Task - Perl extension for CipUX

=head1 SYNOPSIS

  use CipUX::Task;

=head1 DESCRIPTION

Provides the functions for cipux_task_*.

=head2 EXPORT

See CipUX::Task EXPORT variable.

=cut

# +------------------------------------------------------------------------+
# | Global VARS                                                            |
# +------------------------------------------------------------------------+
my $SCRIPT = 'CipUX::Task';
my $DEBUG  = "yes";

my $set = '/usr/bin/cipux_set_value';
my $get = '/usr/bin/cipux_get_value';
my $add = '/usr/bin/cipux_add';
my $del = '/usr/bin/cipux_del';

my $errorlog = "/var/log/cipux/cipux_error.log";


# TODO: get this from cipux.conf with a module!
  my %cipux = ();
  $cipux{'Cipux_Use_Samba'} = `grep Cipux_Use_Samba /etc/cipux/cipux.conf|sed s/Cipux_Use_Samba//`;
  chomp($cipux{'Cipux_Use_Samba'});
  $cipux{'Cipux_Use_Samba'} =~ s/\s+//g;
  $cipux{'Cipux_Use_Samba'} =~ s/=//g;

#  => 0,0                        : no argument to this function
#   from 1, 2, 3, ... counting
#  => 2,0: UID, CipuxFirstname   : 2 arguments = uid and firstname
#  => 2,1: UID, GID; MID         : 2 mandatory arguments, MID is auxilliary
our %task =
  (
    cipux_task_create_object           => '5,6:OID,firstname,lastname,user_node_role,group_node_role;group_type,password,default_CID,snd_skel_Uid,uhquota,usquota,ghquota,gsquota',

    cipux_task_create_user             => '4,7:UID,firstname,lastname,role;password,default_CID,snd_skel_Uid,uhquota,usquota,ghquota,gsquota',
    cipux_task_delete_user             => '1,0:UID',
    cipux_task_list_users              => '0,0:;',
    cipux_task_change_user_password    => '1,1:UID;password',
    cipux_task_backup_user_homedir     => '1,0:UID',
    cipux_task_delete_user_homedir     => '1,0:UID',
    cipux_task_disable_user            => '1,0:UID',
    cipux_task_enable_user             => '1,0:UID',
    cipux_task_restore_user_homedir    => '1,2:UID;homedirpath;skel_UID',
    cipux_task_reset_user_homedir      => '1,2:UID;homedirpath;skel_UID',
    cipux_task_add_user_to_course      => '2,0:CID,member_UID',

    cipux_task_create_course           => '1,5:CID;password,uhquota,usquota,ghquota,gsquota',
    cipux_task_delete_course           => '1,0:CID',
    cipux_task_list_courses            => '0,0:;',
    cipux_task_change_course_password  => '1,1:CID;password',
    cipux_task_disable_course          => '1,0:CID',
    cipux_task_enable_course           => '1,0:CID',

    cipux_task_list_user_members_of_course
                                       => '1,0:CID',
    cipux_task_remove_nonexistent_users_from_all_courses
                                       => '0,0:;',
    cipux_task_remove_user_from_course => '2,0:CID,member_UID',

    cipux_task_create_skel             => '1,1:SID;password',
    cipux_task_delete_skel             => '1,0:SID;',

    cipux_task_create_role             => '1,1:RID;password',
    cipux_task_delete_role             => '1,0:RID;',


     cipux_task_list_group_node_role    => '1,0:role;',
     cipux_task_list_user_node_role     => '1,0:role;',
     cipux_task_list_teachers           => '0,0:;',
     cipux_task_list_students           => '0,0:;',
     cipux_task_list_guests             => '0,0:;',
     cipux_task_list_roles              => '0,0:;',
     cipux_task_list_skels              => '0,0:;',
     cipux_task_list_objects            => '0,0:;',
     cipux_task_list_rpc_functions      => '0,0:;',

  cipux_task_change_cipux_config_variable_value => '2,0:system,value;',

    cipux_task_list_rooms               => '0,0:;',
    cipux_task_create_room              => '5,1:CN,network_address,network_mask,gateway_address,gateway_mask;description',
    cipux_task_delete_room              => '1,0:CN;',
    cipux_task_list_machines            => '0,0:;',
    cipux_task_create_machine           => '1,6:MID;client_number,mac_address,room_name,master,image_slot_number,hardware_type_number',
    cipux_task_delete_machine           => '1,0:MID',
    cipux_task_list_hardware_types      => '0,0:;',
    cipux_task_create_hardware_type     => '4,0:CN,hardware_type_name,hardware_type_protect,hardware_type_nic;',
    cipux_task_delete_hardware_type     => '1,0:CN;',
    cipux_task_list_image_slots         => '0,0:;',
    cipux_task_create_image_slot        => '4,0:CN,image_slot_name,image_slot_protect,image_slot_enabled;',
    cipux_task_delete_image_slot        => '1,0:CN;',
    cipux_task_list_images              => '0,0:;',
    cipux_task_create_image             => '9,0:CN,image_name,image_type,image_protect,image_enabled,image_size,checksum,hardware_type_number,image_slot_number;',
    cipux_task_delete_image             => '1,0:CN;',

    cipux_task_create_applicant         => '3,6:firstname,lastname,role;password,default_CID,snd_skel_Uid,uhquota,usquota,ghquota,gsquota',
    cipux_task_print_apply_for_admission => '1,0:UID',

    cipux_task_disable_user_posix_account   => '1,0:UID',
    cipux_task_disable_user_samba_account   => '1,0:UID',
    cipux_task_disable_course_posix_account => '1,0:CID',
    cipux_task_disable_course_samba_account => '1,0:CID',
    cipux_task_enable_user_posix_account    => '1,0:UID',
    cipux_task_enable_user_samba_account    => '1,0:UID',
    cipux_task_enable_course_posix_account  => '1,0:CID',
    cipux_task_enable_course_samba_account  => '1,0:CID',

    cipux_task_create_examination_course    => '2,8:CID,leader;password,uhquota,usquota,ghquota,gsquota',
    cipux_task_create_examination_user      => '2,8:UID,leader;uhquota,usquota,ghquota,gsquota',

    cipux_task_tell_me_user_firstname       => '1,0:UID',
    cipux_task_tell_me_user_lastname        => '1,0:UID',
    cipux_task_tell_me_user_canonical_name  => '1,0:UID',
    cipux_task_tell_me_user_role            => '1,0:UID',

    cipux_task_change_own_password    => '1,1:UID;password',
    cipux_task_change_user_role       => '1,1:RID.RID',

  );

our %option =
  (
     mode             => 'modes = cgi curses console',
     OID              => 'login: object ID, example for object = user : chkuelker',
     UID              => 'login: user ID, example: chkuelker',
     CID              => 'login: course ID, example: linuxcourse',
     SID              => 'login: skel ID, example: cycle1',
     RID              => 'login: role ID, example: student',
     password         => 'a secret',
     firstname        => 'human first name',
     lastname         => 'human family name',
     role             => 'role, like stutent, teacher, ...',
     user_node_role   => 'role like: account, share, ..',
     group_node_role  => 'role like: student, teacher, skel, ...',
     snd_skel_Uid     => 'secondary skeleton ID: example cycle1',
     default_CID      => 'default course login: course ID, group ID',
     member_UID       => 'member login of a given course: user ID, example: chkuelker',
     uhquota          => 'user hard quota',
     ghquota          => 'group hard quota',
     usquota          => 'user soft quota',
     gsquota          => 'groups soft quota',
     homedirpath      => 'path where all the homedir are, example: /skole/tjener/home0',
     skel_UID         => 'UID for the skell-user, example: student',
     system           => 'system: CAT, CipUX',
     equation         => 'some arbitrary string with a = sign, example: Cat_Feature_FAQ=1',
     CN               => 'CN: canonical name',
     network_address  => 'network address, example 192.168.100.0',
     network_mask     => 'network mask, example 255.255.255.0',
     gateway_address  => 'gateway address, example 192.168.100.254',
     gateway_mask     => 'gateway mask, example 255.255.255.0',
     description      => 'some abitrary string',
     MID              => 'some abitrary string',
     mac_address           => 'the MAC address of the client: example 00:04:B7:71:C1:A5 ',
     room_name             => 'some fancy name for a room',
     client_number         => 'Computer number, last digit of the IP, example: 250',
     master                => 'master mode, TRUE or FALSE, exaple: FALSE',
     image_slot_number     => 'internal CipUX number of the image slot, example: 1',
     hardware_type_number  => 'internal CipUX number for the hardware, example: 1',
     hardware_type_name    => 'some abitrary string',
     hardware_type_protect => 'portection, TRUE or FALSE, example: TRUE',
     hardware_type_nic     => 'a kernel module name for a network interface, example: 3c509x ',
     image_slot_name       => 'some abitrary string',
     image_slot_protect    => 'portection, TRUE or FALSE, example: TRUE',
     image_slot_enabled    => 'in use? TRUE or FALSE, example: FALSE',
     image_name            => 'some abitrary string',
     image_type            => 'the type of the image: tar, dd',
     image_protect         => 'portection, TRUE or FALSE, example: TRUE',
     image_enabled         => 'in use? TRUE or FALSE, example: FALSE',
     image_size            => 'image size',
     checksum              => 'MD5 Checksum string',
  );





# *************************************************************************
# * Auxilliary methodes go here                                           *
# *************************************************************************

=pod

=head1 Auxilliary methodes

This methodes are reserved for internal use only.

=cut

# +------------------------------------------------------------------------+
# |  test_password_existence                                               |
# +------------------------------------------------------------------------+

=pod

=head2 test_password_existence

check if the password is defined and non empty. Do something if we
have no password.

 - if it is defined and non empty return it
 - if it is defined and empty, get a random password
 - if it is not defined and the programm is interactive, promt for new password
 - if it is nit defined and the programm is not interactive, get one from STDIN

B<Syntax:>

  my $password = &test_password_existence($password);

=cut

sub test_password_existence{
  my $password = shift;

  if(defined($password)){
    if($password eq ''){
      $password = &get_random_password();
    }
  }else{
  # if(&i_am_interactive()){
  #$password = &get_password_from_stdin();
  #}else{
     $password = get_password_from_command_line();
   # }
  }
  return $password;
}

# +------------------------------------------------------------------------+
# |  get_password_from_stdin                                               |
# +------------------------------------------------------------------------+

=pod

=head2 get_random_password

Generate a random passoword

B<Syntax:>

  my $password = &random_password();

=cut

sub get_random_password{
  &debug($SCRIPT,"&random_apssword()");

  # see cookbook, de <55>
  my @chars = ('A'..'Z','a'..'z',0..9,qw(! @ $ % &));
  my $password = join("",@chars[map{rand@chars}(1..8)]);

  #&debug($SCRIPT,"=> return \$password [$password]");
  return $password;
}

# +------------------------------------------------------------------------+
# |  get_password_from_stdin                                               |
# +------------------------------------------------------------------------+
sub get_password_from_stdin{
  my $password = undef;
  my $stdin = undef;
  if(defined($stdin = <STDIN>)){
    $password = $stdin;
    chomp($password);
    &log($SCRIPT,"- got password from STDIN");
  }
  return $password;
}

# +------------------------------------------------------------------------+
# |  i_am_interactive                                                      |
# +------------------------------------------------------------------------+
sub i_am_interactive {
  # cookbook 15.2. Testing Whether a Program Is Running Interactively
  return -t STDIN && -t STDOUT;
}

# +------------------------------------------------------------------------+
# | debug                                                                  |
# +------------------------------------------------------------------------+
sub debug{
  my $script = shift;
  my $message = shift;

  if($DEBUG eq 'yes'){
    my $path = "/var/log/cipux";
    if(!-d $path){mkdir($path,700)}
    my $logfile=$path."/".$script.".log";
    open(DEBUG,">>$logfile") or die "Can't open logfile '$logfile'";
    #my $date = `date`;
    #chomp($date);
    print DEBUG "$script: $message\n";
    close(DEBUG);
  }
}
# +------------------------------------------------------------------------+
# | log                                                                    |
# +------------------------------------------------------------------------+
sub log{
  my $script = shift;
  my $message = shift;

  my $path = "/var/log/cipux";
  if(!-d $path){mkdir($path,700)}
  my $logfile=$path."/".$script.".log";
  open(LOG,">>$logfile") or die "Can't open logfile '$logfile'";
  #my $date = `date`;
  #chomp($date);
  print LOG "$script: $message\n";
  close(LOG);
}
# +------------------------------------------------------------------------+
# | task_exec_cmd_writing                                                  |
# +------------------------------------------------------------------------+
# used by
# - cipux_task_change_user_password
# - cipux_task_enable_user_posix_account
# - cipux_task_disable_user_posix_account
#
sub task_exec_cmd_writing{
  # my @cmd_aref = ($set,"-u",$uid,"-e","userPassword","-V");
  # &task_exec_cmd_writing($password,\@cmd_aref);
  #
  # for security reason we do:
  # - fork the execution with exec
  # - write the data with a internal perl pipe to the program,
  #   not by command line option
  # - escape the execution from the shell
  my $data     = shift; # e.g.: "{crypt}$crypt_password"
  # @$cmd_aref = (command, option, value, option, value, ....)
  my $cmd_aref = shift;  # my @$cmd_aref = ($set,"-u",$uid,"-e","userPassword","-V");

  foreach(@$cmd_aref){
     # the password is in $data, so no password will be printed
     # not even a hash
     &debug($SCRIPT,"- execute secure write with parameter [$_]");
  }

  # 19.6. Executing Commands Without Shell Escapes (cookbook)
  my $pid = undef;
  die "cannot fork: $!" unless defined($pid = open(KID_TO_WRITE, "|-"));
  $SIG{ALRM} = sub{
          my $date = `date`;
          &log("$date CipUX::Task ERROR: [$$cmd_aref[0]] pipe broke!");
          die "whoops, $$cmd_aref[0] pipe broke"
                   };
  if($pid){  # parent
    &debug($SCRIPT,"- [parent] write data to child");
    print KID_TO_WRITE $data;
    #for (@data){ print KID_TO_WRITE $_ }
    close(KID_TO_WRITE)              or warn "kid [$$cmd_aref[0]]exited $?";
  }else{  # child
    &debug($SCRIPT,"- [child] recieve data from parent for [$$cmd_aref[0]]");
    # reconfigure, then
    exec { $$cmd_aref[0] } @$cmd_aref
      or do{
            my $date = `date`;
            &log("$date CipUX::Task ERROR: can not execute command [$$cmd_aref[0]]: $!");
            die "can't exec program: $!";
          }
  }
}

# +------------------------------------------------------------------------+
# | split_to_first_column                                                  |
# +------------------------------------------------------------------------+
# used by
# - cipux_task_list_users
# - cipux_task_list_courses
sub split_to_first_column{
  my $out = shift;
  my @return = ();
   foreach (@$out){
    chomp();
    #$string]=~ s/\t/HALLO/g;
    my ($u1,$u2) = split("\t",$_);
    if(defined($u1)){# if it is TWO columns
      push(@return,$u1);
    }else{# it may be only ONE column
      push(@return,$_);
    }
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | get_passwords_from_command_line                                        |
# +------------------------------------------------------------------------+
# used by
# - cipux_task_change_user_password
# - cipux_task_change_course_password (indirect)
sub get_password_from_command_line{
  my $password  = '';
  my $password1 = '';
  my $password2 = '';

  print "please enter password: ";
  system "stty -echo";
  chomp($password1=<STDIN>);
  print "\n";
  system "stty echo";
  print "please repeat password once again: ";
  system "stty -echo";
  chomp($password2=<STDIN>);
  print "\n";
  system "stty echo";
  if($password1 eq ""){
     print "ERROR: empty password given!\n";
     exit(1);
  }
  if($password1 ne $password2){
     print "ERROR: passwords do not match!\n";
     exit(1);
  }
  $password = $password1;
  return $password;
}

# +------------------------------------------------------------------------+
# | taskusage                                                              |
# +------------------------------------------------------------------------+
sub taskusage{
  my $DEBUG = 0;
  #my $self = shift;
  my $caller = shift;
  my $n = 0;

   print "CipWorX CipUX CiBot Task \n";
   print '-'x76 . "\n";
   print "USAGE:\n\n";
   my ($args_no,$args) = split(':',$task{$caller},2);
   my ($man,$aux) = split(',',$args_no,2);
   print "  $caller ";
   if(defined($args)){
     my ($mandatory,$optional) = split(';',$args,2);
     print "* mandatory [$mandatory] optional [$optional]\n" if $DEBUG;
     print "[OPTIONS]\n\n";
     print "EXPLANION:\n\n";
     if($man > 0){# from 0 to x, mandatory
       my(@mandatory) = split(',',$mandatory);
       #push(@mandatory,$mandatory) if defined($mandatory);
       if($#mandatory >= 0){# from -1 to x
         print "  * $man mandatory options:\n";
         foreach (@mandatory){
           print "      $_ ($option{$_})\n";
         }
         print "\n";
       }
     }
     if($aux > 0){ # from 0 to x, auxilliary
       my(@optional)  = split(',',$optional);
       #push(@optional,$optional) if defined($optional);
       if($#optional >= 0){# from -1 to x
         print " * $aux auxilliary options:\n";
         foreach (@optional){
           print "      $_ ($option{$_})\n";
         }
         print "\n";
       }
     }
   }else{# if args NOT defined (nor arguments)
     print " \n";
   }
   print "  * 2 other options:\n";
   print "      -h (this page)      \n";
   print "      --help (this page)   \n\n";
   print "EXAMPLES:\n\n";
   if($man ne 0){# we should use only man options
     my ($mandatory,$optional) = split(';',$args,2);
     $mandatory =~ s/,|;/ /g;
     $n++;
     print "  $n) $caller $mandatory\n";
     if($args =~ m/password/){
         $n++;
         print "     if using without password, you will be ask for password\n";
         print "  $n) echo 'password' | $caller $mandatory -\n";

     }
     if($aux ne 0){
       $n++;
       $args =~ s/,|;/ /g;
       print "  $n) $caller $args\n";

     }
   }else{
     $n++;
     print "  $n) $caller\n";
   }
     $n++;
     print "  $n) $caller -h\n";
     $n++;
     print "  $n) $caller --help\n";
     print '-'x76 . "\n";
}

# *************************************************************************
# * Preloades methodes go here                                            *
# *************************************************************************

=pod

=head1 Preload methodes

This methodes are are exported and may be use by the cipux_task_*
scripts.

=cut

# +------------------------------------------------------------------------+
# | cipux_task_add_member_to_course                                        |
# +------------------------------------------------------------------------+

=pod

=head2 cipux_task_add_user_to_course

This script adds a given user to given course

B<Syntax:>

see cipux_task_add_user_to_course -h

=cut

sub cipux_task_add_user_to_course{
  # &cipux_task_add_user_to_course('cgi',$course,$member);
  my $mode       = shift;
  my $cid        = shift;
  my $member_uid = shift;

  system("$set -a -c '$cid' -e 'memberUid' -v '$member_uid'");
}

# +------------------------------------------------------------------------+
# | cipux_task_backup_user_homedir                                         |
# +------------------------------------------------------------------------+
sub cipux_task_backup_user_homedir{
  my $mode      = shift;
  my $uid       = shift;
  my $backupdir = "/skole/backup";# TODO from config file
  my $homepath  = "/skole/tjener/home0";
  my $homedir   = "/skole/tjener/home0/$uid"; # TODO from config file
  # TODO: ckeck this, and make this a config part
  my $date = `date +%F+%T`;
  chomp($date);

  mkdir $backupdir if not -d $backupdir;

  my $cmd = "cd $homepath && tar cvjf $backupdir/backup_homedir_".$uid."_".$date.".tar.bz2 $uid";
  #print "CMD [$cmd]\n";

  # create archive: UID/*
  if(-d "$homepath/$uid"){
    system($cmd);
  }
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_user_homedir                                         |
# +------------------------------------------------------------------------+
sub cipux_task_delete_user_homedir{
  my $mode      = shift;
  my $uid       = shift;
  my $basedir   = "/skole/tjener/home0/"; # TODO from config file
  my $homedir   = "/skole/tjener/home0/$uid"; # TODO from config file

  if(defined($uid)){
    if($uid ne ''){
      if(length($uid) > 1){
        if($homedir ne '/'){
          if("$basedir" ne "$homedir"){
	    if(-d $homedir){
              system("rm -r $homedir");
	    }
	  }
	}
      }
    }
  }
}

# +------------------------------------------------------------------------+
# | cipux_task_change_user_role                                            |
# +------------------------------------------------------------------------+
sub cipux_task_change_user_role{
  my $mode        = shift;
  my $uid         = shift;
  my $role        = shift; # e.g.: teacher
  if($role eq "student"
    or $role eq "teacher"
    or $role eq "guest"
    or $role eq "tutor"
    or $role eq "examenee"
    or $role eq "assistant"
    or $role eq "skel"
    or $role eq "admin"
    or $role eq "none"){
    my $cmd = "cipux_set_value -c $uid -c cipuxRole -v $role";
    system("$cmd");
  }
}

# +------------------------------------------------------------------------+
# | cipux_task_change_user_password                                        |
# +------------------------------------------------------------------------+
sub cipux_task_change_user_password{
  # used by cipux_task_change_course_password
  my $mode         = shift; # cgi, curses, console, rpc
  my $uid          = shift;
  my $cml_password = shift;
  my $password = '';
  # this was the old style, on terminal server, this is not apropriate
  # system("/usr/bin/cipux_set_password.pl -u '$uid' -p '$password'");
  if(not defined($cml_password) and $mode eq 'console'){
    &debug($SCRIPT,"- Password from console");
    $password = get_password_from_command_line();
  }elsif(defined($cml_password)){
    if($cml_password eq '-'){
      &debug($SCRIPT,"- Password from pipe  given");
      $password = get_password_from_stdin();
    }else{
      &debug($SCRIPT,"- Password from paramater given");
      $password = $cml_password;
    }
  }else{
    &debug($SCRIPT,"- WARNING: got not defined password, using empty password");
  }
  if($cipux{Cipux_Use_Samba} eq "yes"){
    &debug($SCRIPT,"- set samba password for $uid (because Cipux_Use_Samba = yes)");
    # This also will add the ldap values
    open(CMD,"| smbpasswd -s $uid >>/var/log/cipux/CipUX::Task.log 2>&1");
    print CMD "$password\n$password\n";
    close(CMD);
  }else{
    &debug($SCRIPT,"- do not set samba password for ID [$uid] (because Cipux_Use_Samba = no)");
  }
  &debug($SCRIPT,"- set POSIX password for ID [$uid]");
  my $modsalt = join '', ('.','/',0..9,'A'..'Z','a'..'z')[rand 64,rand 64];
  my $crypt_password = crypt($password,$modsalt);

  # if a normal user can browse password hashes this is save enough,
  # but do not change this to clear passwords!
  #system("$set -u '$uid' -e 'userPassword' -v '{crypt}$crypt_password'");
  #system("$set -c '$uid' -e 'userPassword' -v '{crypt}$crypt_password'");

  # but anyway on Terminal servers we do this way:
  my @cmd = ($set,"-u",$uid,"-e","userPassword","-V");
  &task_exec_cmd_writing("{crypt}$crypt_password",\@cmd);
  @cmd = ($set,"-c",$uid,"-e","userPassword","-V");
  &task_exec_cmd_writing("{crypt}$crypt_password",\@cmd);

  #&debug($SCRIPT,"- Set password for [$uid]");
}
# +------------------------------------------------------------------------+
# | cipux_task_change_own_password                                         |
# +------------------------------------------------------------------------+
# this is basical the same as cipux_task_change_user_password but to 
# distinguish this by means of ACL of the CipUX XML-RPC server, we have to
# define this function, because a user should only be able to change his own
# password. (This is even true, if he writes his own XML-RPC client.)
sub cipux_task_change_own_password{
  my $mode         = shift; # cgi, curses, console, rpc
  my $uid          = shift;
  my $password     = shift;
  if($mode eq 'console'){
    if(defined($ENV{USER})){
      if($ENV{USER} eq $uid){
        # this is not realy nessessary, because the security layer of pam will 
        # prevent this, but we like to have a nice error message.
        &cipux_task_change_user_password($mode,$uid,$password);
      }else{
       print "ERROR: You can't change passwords of other users with this function!\n";
      }
    }
  }
  if($mode eq 'rpc'){
    &cipux_task_change_user_password($mode,$uid,$password);
  }
}

# +------------------------------------------------------------------------+
# | cipux_task_change_course_password                                        |
# +------------------------------------------------------------------------+
sub cipux_task_change_course_password{
  my $mode         = shift; # cgi, curses, console, rpc
  my $cid          = shift;
  my $cml_password = shift;

  &cipux_task_change_user_password($mode,$cid,$cml_password);
}

# +------------------------------------------------------------------------+
# | cipux_task_reset_user_homedir                                          |
# +------------------------------------------------------------------------+
sub cipux_task_reset_user_homedir{
  # 2 = reset = delete and create
  my $mode         = shift;
  # man
  my $uid          = shift;
  # opt
  my $homedirpath  = shift;
  my $skeluser     = shift;

  $homedirpath = '/skole/tjener/home0' if not defined($homedirpath);
  if(not defined($skeluser)){
  # TODO: check the SYSTEM
    $skeluser = `$get -c $uid -e cipuxRole`;
    chomp($skeluser);
  }

  #  2    /skole/tjener/home0 teachers wbolte
  # MODE HOMEDIRPATH         SKELUSER USER
  &debug($SCRIPT,"cipux_task_reset_user_homedir: /usr/bin/cipux_refine_homedir reset '$homedirpath' '$skeluser' '$uid'");
  system("/usr/bin/cipux_refine_homedir reset '$homedirpath' '$skeluser' '$uid'");

  if($skeluser eq 'student'){
    my $forbachskel = `$get -u $uid -e cipuxSkeletonUid`;
    chomp($forbachskel);
    &debug($SCRIPT,"cipux_task_reset_user_homedir: role is stutent: using forbachskel [$forbachskel]");
    # WARNING: we must use restore, or onle the last refine will remain
     system("/usr/bin/cipux_refine_homedir restore '$homedirpath' '$forbachskel' '$uid'");

  }

}

# +------------------------------------------------------------------------+
# | cipux_task_restore_user_homedir                                          |
# +------------------------------------------------------------------------+
sub cipux_task_restore_user_homedir{
  # 1 = restore = rsync only the config data
  my $mode     = shift;
  # man
  my $uid      = shift;
  # opt
  my $homedirpath  = shift;
  my $skeluser = shift;

  $homedirpath = '/skole/tjener/home0' if not defined($homedirpath);
  if(not defined($skeluser)){
  # TODO: check the SYSTEM
    $skeluser = `$get -c $uid -e cipuxRole`;
    chomp($skeluser);
  }

  #  1    /skole/tjener/home0 teachers wbolte
  # MODE  HOMEDIRPATH         SKELUSER USER
  &debug($SCRIPT,"cipux_task_reset_user_homedir: /usr/bin/cipux_refine_homedir restore '$homedirpath' '$skeluser' '$uid'");
  system("/usr/bin/cipux_refine_homedir restore '$homedirpath' '$skeluser' '$uid'");

  if($skeluser eq 'student'){
    my $forbachskel = `$get -u $uid -e cipuxSkeletonUid`;
    chomp($forbachskel);
    &debug($SCRIPT,"cipux_task_restore_user_homedir: role is student: using forbachskel [$forbachskel]");
     system("/usr/bin/cipux_refine_homedir restore '$homedirpath' '$forbachskel' '$uid'");

  }
}

# +------------------------------------------------------------------------+
# | cipux_task_create_course                                               |
# +------------------------------------------------------------------------+
sub cipux_task_create_course{
 # &cipux_task_create_course('cgi',$password,$course,$hquota,$squota,$type);
  my $mode       = shift; # cgi
  # man:
  my $id         = shift; # grammar   group: cn, gid, course_login; user: uid, login
  # opt 1:
  my $password   = shift; # secret
  my $uhquota    = shift; # 1000
  my $usquota    = shift; # 800
  # opt 2:
  my $gsquota    = shift; # 1800
  my $ghquota    = shift; # 2000
  #  default
  my $role1       = 'share';
  my $role2       = 'course';
  my $group_type  = 'public';
  my $homedirpath = '/skole/tjener/home0';
  # aux:
  #my $group_type = shift; # school_class   private
  #my $role       = shift; # course         teacher
  # my $firstname  = shift; # cipux
  # my $lastname   = shift; # course
  # my $role       = shift; # course
  # my $group      = shift;
  # my $course     = shift;


  $password= &test_password_existence($password);

  # create group VORBILD
  my $c_attr = " --attribute cn='$id'";
  $c_attr   .= " --attribute groupType='$group_type' ";
  $c_attr   .= " --attribute cipuxSoftQuota='$gsquota'" if defined($gsquota);
  $c_attr   .= " --attribute cipuxHardQuota='$ghquota'" if defined($ghquota);
  $c_attr   .= " --attribute cipuxRole='$role2'";
  $c_attr   .= " --attribute cipuxInternetStatus=accept";
  $c_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";

  my @c_attr = ($add, "-c", "-W", "--attribute", "cn=$id",
                                  "--attribute", "groupType=$group_type",
                                  "--attribute", "cipuxRole=$role2",
                                  "--attribute", "cipuxInternetStatus=accept",
                                  "--attribute", "cipuxRemark=created by CipUX::Task ($mode)");
  if(defined($gsquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxSoftQuota=$gsquota");
  }
  if(defined($ghquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxHardQuota=$ghquota");
  }

  #system("/usr/bin/cipux_add -c -w '$password' $c_attr");
  # exec commad with pipe, give string, ref to array
  &task_exec_cmd_writing($password,\@c_attr);

  # create login
  my $u_attr = " --attribute uid='$id'";
  $u_attr   .= " --attribute cipuxSoftQuota='$usquota'" if defined($usquota);
  $u_attr   .= " --attribute cipuxHardQuota='$uhquota'" if defined($uhquota);
  $u_attr   .= " --attribute cipuxRole='$role1'";
  $u_attr   .= " --attribute cipuxInternetStatus=accept";
  $u_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";
  $u_attr   .= " --attribute cipuxFirstname='CipUX'";
  $u_attr   .= " --attribute cipuxLastname='course'";

  my @u_attr = ($add, "-u", "-W",
                     "--attribute", "uid=$id",
                     "--attribute", "cipuxRole=$role1",
                     "--attribute", "cipuxInternetStatus=accept",
                     "--attribute", "cipuxRemark=created by CipUX::Task ($mode)",
                     "--attribute", "cipuxFirstname=CipUX",
                     "--attribute", "cipuxLastname=course");

  if(defined($usquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxSoftQuota=$usquota");
  }
  if(defined($uhquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxHardQuota=$uhquota");
  }

  #system("/usr/bin/cipux_add -u -w '$password' $u_attr");
  # exec commad with pipe, give string, ref to array
  &task_exec_cmd_writing($password,\@u_attr);

  # make only an refinement to skel if this is one of the main roles

  #  2    /skole/tjener/home0 teachers wbolte
  #                                   MODE HOMEDIRPATH   SKELUSER USER
  system("/usr/bin/cipux_refine_homedir reset '$homedirpath' '$role2' '$id'");
  return 0
}

# +------------------------------------------------------------------------+
# | cipux_task_create_examination_course                                   |
# +------------------------------------------------------------------------+
sub cipux_task_create_examination_course{
  my $mode       = shift; # cgi
  # man
  my $id         = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $leader     = shift; # bibeutlin
  # opt
  my $password   = shift; # secret
  my $uhquota    = shift; # 1000
  my $usquota    = shift; # 800
  my $group_type = 'school_exam'; # school_exam
  my $role       = 'examination'; # examination
  #my $course     = shift;
  my $ghquota    = shift; # group hard quota
  my $gsquota    = shift; # group soft quota

  #system("/usr/bin/cipux_add -c -w '$password' --attribute cn='$course' --attribute groupType='school_exam' --attribute cipuxSoftQuota='$usquota' --attribute cipuxHardQuota='$uhquota' --attribute cipuxGroupLeaderUid='$leader' --attribute cipuxRole='examination' --attribute cipuxInternetStatus=accept");

  $password= &test_password_existence($password);

  #NACHBILD 1
  # changes userquota to groupquota (2006-04-09)
  my @c_attr = ($add, "-c", "-W",
                     "--attribute", "cn=$id",
                     "--attribute", "groupType=school_exam",
                     "--attribute", "cipuxGroupLeaderUid=$leader",
                     "--attribute", "cipuxRole=examination",
                     "--attribute", "cipuxInternetStatus=accept",
                     "--attribute", "cipuxRemark=created by CipUX::Task ($mode)",
                     "--attribute", "cipuxFirstname=CipUX",
                     "--attribute", "cipuxLastname=course");

  # if we have grou quota, we use this, but if not use user quota if it is aviable
  if(defined($gsquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxSoftQuota=$gsquota");
  }elsif(defined($usquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxSoftQuota=$usquota");
  }

  # if we have grou quota, we use this, but if not use user quota if it is aviable
  if(defined($ghquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxHardQuota=$ghquota");
  }elsif(defined($uhquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxHardQuota=$uhquota");
  }

  &task_exec_cmd_writing($password,\@c_attr);

  # create login
  #system("/usr/bin/cipux_add -u -w '$password' --attribute cipuxHardQuota='$uhquota' --attribute cipuxSoftQuota='$usquota)' --attribute cipuxFirstname='cipux' --attribute cipuxLastname='examination' --attribute cipuxRole='course' --attribute cipuxInternetStatus=accept --attribute uid='$course' ");
  my @u_attr = ($add, "-u", "-W",
                     "--attribute", "uid=$id",
                     "--attribute", "cipuxRole=course",
                     "--attribute", "cipuxInternetStatus=accept",
                     "--attribute", "cipuxRemark=created by CipUX::Task ($mode)",
                     "--attribute", "cipuxFirstname=CipUX",
                     "--attribute", "cipuxLastname=examination");

  if(defined($usquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxSoftQuota=$usquota");
  }
  if(defined($uhquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxHardQuota=$uhquota");
  }

  &task_exec_cmd_writing($password,\@u_attr);
}

# +------------------------------------------------------------------------+
# | cipux_task_create_skel                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_skel{
  my $mode = shift;
  # mandatory
  my $login = shift;
  # optional
  my $password =shift;

  &cipux_task_create_user('cgi',
    $login,
    'CipUX',
    "Skel UID",
    'skel',
    $password);

  # TODO make a create_object subroutine
  my $cmd = "cipux_set_value -c $login -e groupType -v shared";
  #print "cipux_set_value -c $login -e groupType -v shared";
  `$cmd`;
  # TODO make password optional


}
# +------------------------------------------------------------------------+
# | cipux_task_create_role                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_role{
  my $mode = shift;
  # mandatory
  my $login = shift;
  # optional
  my $password =shift;

  &cipux_task_create_user('cgi',
    $login,
    'CipUX',
    "Role",
    'none',
    $password);

  # TODO make a create_object subroutine
  my $cmd = "cipux_set_value -c $login -e groupType -v shared";
  `$cmd`;
  $cmd = "cipux_set_value -c $login -e cipuxRole -v skel";
  `$cmd`;
  $cmd = "cipux_set_value -u $login -e cipuxRole -v role";
  `$cmd`;

  # TODO make password optional


}

# +------------------------------------------------------------------------+
# | cipux_task_create_applicant                                            |
# +------------------------------------------------------------------------+
sub cipux_task_create_applicant{
  my $mode = shift;
  # mandatory
  my $login = "";
  my $firstname    = shift; # cipux
  my $lastname     = shift; # course
  my $role         = shift; # student
  my $password     = shift; # secret
  # opt 1:
  my $default_CID  = shift; # chemie   memberUid # default group
  my $snd_skel_Uid = shift; # secondary skeleton ID: cycle1
  my $uhquota      = shift; # 1000
  my $usquota      = shift; # 800
  # opt 2:
  my $ghquota      = shift; # 2000
  my $gsquota      = shift; # 1800

  my $length=128;
  my $first=2;
  my $last=$length-$first;



  my $string1 = substr($firstname,0,$first);
  my $string2 = substr($lastname,0,$last);
  $login = substr($string1.$string2,0,$length);
  $login=lc($login);
  my @user =&cipux_task_list_users($mode);
  my %user = ();
  foreach(@user){
    $user{$_}=1;
  }
  my $z='1';
  # TODO: The login will get "shorter", the length of $z
  # is not always correct
  while(defined($user{$login})){
	  #print "test for login [$login] z [$z]\n";
    if(defined$user{$login}){
      if($z eq '1'){
        $login = $login.$z;
      }else{
        my $l = length($z);
        $login = substr($login,0,-$l);
        $login .=$z;
      }
      $z++;
    }
    }

# print "LOGIN [$login]\n";


 $mode = 'task' if not defined($mode);

  &cipux_task_create_user($mode,$login,$firstname,$lastname,$role,$password,$default_CID,$snd_skel_Uid,$uhquota,$usquota,$ghquota,$gsquota);
  &cipux_task_disable_user($mode,$login);
 my $cmd = "cipux_set_value -u $login -e cipuxStatus -v pending";
  `$cmd`;
  return $login;

}

# +------------------------------------------------------------------------+
# | cipux_task_create_user                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_user{
  # &cipux_task_create_user('cgi',$password,$login,$hquota,$squota,$role,$firstname,$lastname);
  my $mode         = shift; # cgi
  # man:
  my $id           = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $firstname    = shift; # cipux
  my $lastname     = shift; # course
  my $role         = shift; # student
  my $password     = shift; # secret
  # opt 1:
  my $default_CID  = shift; # chemie   memberUid # default group
  my $snd_skel_Uid = shift; # secondary skeleton ID: cycle1
  my $uhquota      = shift; # 1000
  my $usquota      = shift; # 800
  # opt 2:
  my $ghquota      = shift; # 2000
  my $gsquota      = shift; # 1800
  #my $group_type  = shift; # private
  #my $group       = shift;# chemie   memberUid # default group
  my $group_type   = 'private';
  my $dn           = "ou=People,dc=skole,dc=skolelinux,dc=no";


  my $args = join(' ',@_);
  &debug($SCRIPT,'cipux_task_create_user: $id,$firstname,$lastname,$role');
  &debug($SCRIPT,' ');


  if($role eq "student"
    or $role eq "teacher"
    or $role eq "guest"
    or $role eq "tutor"
    or $role eq "examenee"
    or $role eq "assistant"
    or $role eq "skel"
    or $role eq "admin"
    or $role eq "none"){

    # add course
    my $c_attr = "  --attribute cn='$id'";
    $c_attr   .= " --attribute groupType='$group_type' ";
    $c_attr   .= " --attribute cipuxSoftQuota='$gsquota'" if defined($gsquota);
    $c_attr   .= " --attribute cipuxHardQuota='$ghquota'" if defined($ghquota);
    $c_attr   .= " --attribute cipuxRole='$role'";
    $c_attr   .= " --attribute cipuxInternetStatus=accept";
    $c_attr   .= " --attribute memberUid='$id'";
    # $c_attr   .= " --attribute member='uid=$id,$dn'"; # QUESTION: not more in sage debian-edu?
    $c_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";
    #system("/usr/bin/cipux_add -c -w '$password' $c_attr");

# NACHBILD2
    $password= &test_password_existence($password);
    my @c_attr = ($add, "-c", "-W", "--attribute", "cn=$id",
                                    "--attribute", "groupType=$group_type",
                                    "--attribute", "cipuxRole=$role",
                                    "--attribute", "cipuxInternetStatus=accept",
                                    "--attribute", "memberUid=$id",
                                    "--attribute", "cipuxRemark=created by CipUX::Task ($mode)");

  if(defined($gsquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxSoftQuota=$gsquota");
  }
  if(defined($ghquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxHardQuota=$ghquota");
  }

  #system("/usr/bin/cipux_add -c -w '$password' $c_attr");
  # exec commad with pipe, give string, ref to array
  &task_exec_cmd_writing($password,\@c_attr);



    # add user
    my $u_attr = " --attribute uid='$id'";
    $u_attr   .= " --attribute cipuxSoftQuota='$usquota'" if defined($usquota);
    $u_attr   .= " --attribute cipuxHardQuota='$uhquota'" if defined($uhquota);
    $u_attr   .= " --attribute cipuxSkeletonUid='$snd_skel_Uid'" if defined($snd_skel_Uid);
    $u_attr   .= " --attribute cipuxRole='account'"; # policy 2005-08-05
    $u_attr   .= " --attribute cipuxInternetStatus=accept";
    $u_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";
    $u_attr   .= " --attribute cipuxFirstname='$firstname'";
    $u_attr   .= " --attribute cipuxLastname='$lastname'";

    #system("/usr/bin/cipux_add -u -w '$password' $u_attr");

   my @u_attr = ($add, "-u", "-W",
                     "--attribute", "uid=$id",
                     "--attribute", "cipuxRole=account",
                     "--attribute", "cipuxInternetStatus=accept",
                     "--attribute", "cipuxRemark=created by CipUX::Task ($mode)",
                     "--attribute", "cipuxFirstname=$firstname",
                     "--attribute", "cipuxLastname=$lastname");
                     # cipuxRole=account <= policy 2005-08-05
  if(defined($usquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxSoftQuota=$usquota");
  }
  if(defined($uhquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxHardQuota=$uhquota");
  }
  if(defined($snd_skel_Uid)){
   push(@u_attr,"--attribute");
   push(@u_attr,"cipuxSkeletonUid=$snd_skel_Uid");
  }
    # exec commad with pipe, give string, ref to array
    &task_exec_cmd_writing($password,\@u_attr);

    # normal skeleton
    if($role ne "none"){
      my $cmd = "/usr/bin/cipux_refine_homedir restore '/skole/tjener/home0' $role $id";
      &debug($SCRIPT,'* normal skeleton: $cmd');
      system($cmd);
    }

    # Forbach skeleton
    if(defined($snd_skel_Uid)){
      my $cmd = "/usr/bin/cipux_refine_homedir restore '/skole/tjener/home0' $snd_skel_Uid $id";
      &debug($SCRIPT,'* forbach skeleton: $cmd');
      system($cmd);
    }

    # add to user to role group
    system("$set -a -c '$role' -e memberUid -v '$id'");
    #system("$set -a -c '$role' -e member -v 'uid=$id,$dn'");#QUESTION: not more in sage debian-edu?

    # add user to default class
    if(defined($default_CID)){
      # add to default group (for class, or exam)
      system("$set -a -c '$default_CID' -e memberUid -v '$id'");
      #system("$set -a -c '$default_CID' -e member -v 'uid=$id,$dn'");#QUESTION: not more in sage debian-edu?
    }

  }else{
    print "ERROR: can't create role $role!\n" if $mode eq 'console' ;
  }

  # '$password' '$login' '$firstname '$lastname' '$role' '$uhquota' '$usquota' '$ghquota' 'gsquota' '$group'
  #system("/usr/bin/cipux_add -c -w '$password' --attribute cn='$id' --attribute groupType='private' --attribute cipuxSoftQuota='$usquota' --attribute cipuxHardQuota='$uhquota' --attribute cipuxGroupLeaderUid='$leader' --attribute cipuxRole='examination' --attribute cipuxInternetStatus=accept");
  #system("/usr/bin/cipux_add -c -w '$password' --attribute cn='$id' --attribute groupType='private' --attribute cipuxSoftQuota='$usquota' --attribute cipuxHardQuota='$uhquota' --attribute --attribute cipuxRole='student' --attribute cipuxInternetStatus=accept");

  # create login
  #system("/usr/bin/cipux_add -u -w '$password' --attribute cipuxHardQuota='$uhquota' --attribute cipuxSoftQuota='$usquota)' --attribute cipuxFirstname='cipux' --attribute cipuxLastname='examination' --attribute cipuxRole='course' --attribute cipuxInternetStatus=accept --attribute uid='$id' ");

  # add user to role group (ROLE+s)
  #echo "$set -a -c $role.'s' -e memberUid -v '$id'";
  #system("$set -a -c $role -e memberUid -v '$id'");

  # add user to primary group? TODO: neccessary?
  #echo "$set -a -c '$id' --e memberUid -v '$id'";
  #$set -a -c '$id' --e memberUid -v '$id'
  #system("$set -a -c 'id' --e memberUid -v '$id'");

  # add user to default group TODO: if its a student
  #echo "$set -a -c '$group' --e memberUid -v '$id'";
  #$set -a -c '$group' --e memberUid -v '$id'

}

# +------------------------------------------------------------------------+
# | cipux_task_create_object                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_object{
  my $mode         = shift; # cgi
  # man:
  my $id           = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $firstname    = shift; # cipux     cipux     cipux
  my $lastname     = shift; # user      role      course
  my $urole        = shift; # account | role    | share
  my $grole        = shift; # student | skel    |
  my $gtype        = shift; # private | shared  | public
  my $password     = shift; # secret
  # opt 1:
  my $default_CID  = shift; # chemie   memberUid # default group
  my $snd_skel_Uid = shift; # secondary skeleton ID: cycle1
  my $uhquota      = shift; # 1000
  my $usquota      = shift; # 800
  # opt 2:
  my $ghquota      = shift; # 2000
  my $gsquota      = shift; # 1800
  my $dn           = "ou=People,dc=skole,dc=skolelinux,dc=no";

  &debug($SCRIPT,'cipux_task_create_object($id,$firstname,$lastname,$urole,$grole,$gtype)');
  &debug($SCRIPT,' ');


  if($grole eq "student"
    or $grole eq "teacher"
    or $grole eq "guest"
    or $grole eq "tutor"
    or $grole eq "examenee"
    or $grole eq "assistant"
    or $grole eq "skel"
    or $grole eq "admin"
    or $grole eq "none"){

    # add course
    my $c_attr = "  --attribute cn='$id'";
    $c_attr   .= " --attribute groupType='$gtype' ";
    $c_attr   .= " --attribute cipuxSoftQuota='$gsquota'" if defined($gsquota);
    $c_attr   .= " --attribute cipuxHardQuota='$ghquota'" if defined($ghquota);
    $c_attr   .= " --attribute cipuxRole='$grole'";
    $c_attr   .= " --attribute cipuxInternetStatus=accept";
    $c_attr   .= " --attribute memberUid='$id'";
    # $c_attr   .= " --attribute member='uid=$id,$dn'"; # QUESTION: not more in sage debian-edu?
    $c_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";
     my @c_attr = ($add, "-c", "-W", "--attribute", "cn=$id",
                                  "--attribute", "groupType=$gtype",
                                  "--attribute", "cipuxRole=$grole",
                                  "--attribute", "cipuxInternetStatus=accept",
                                  "--attribute", "memberUid=$id",
                                  "--attribute", "cipuxRemark=created by CipUX::Task ($mode)");
  if(defined($gsquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxSoftQuota=$gsquota");
  }
  if(defined($ghquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxHardQuota=$ghquota");
  }

    # NACHBLD2
    #print "/usr/bin/cipux_add -c -w '$password' $c_attr\n";
    #system("/usr/bin/cipux_add -c -w '$password' $c_attr");

   # exec commad with pipe, give string, ref to array
  &task_exec_cmd_writing($password,\@c_attr);

    # add user
    my $u_attr = " --attribute uid='$id'";
    $u_attr   .= " --attribute cipuxSoftQuota='$usquota'" if defined($usquota);
    $u_attr   .= " --attribute cipuxHardQuota='$uhquota'" if defined($uhquota);
    $u_attr   .= " --attribute cipuxSkeletonUid='$snd_skel_Uid'" if defined($snd_skel_Uid);
    $u_attr   .= " --attribute cipuxRole='$urole'";
    $u_attr   .= " --attribute cipuxInternetStatus=accept";
    $u_attr   .= " --attribute cipuxRemark='created by CipUX::Task ($mode)'";
    $u_attr   .= " --attribute cipuxFirstname='$firstname'";
    $u_attr   .= " --attribute cipuxLastname='$lastname'";
  my @u_attr = ($add, "-u", "-W",
                     "--attribute", "uid=$id",
                     "--attribute", "cipuxRole=$urole",
                     "--attribute", "cipuxInternetStatus=accept",
                     "--attribute", "cipuxRemark=created by CipUX::Task ($mode)",
                     "--attribute", "cipuxFirstname=$firstname",
                     "--attribute", "cipuxLastname=$lastname");
if(defined($usquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxSoftQuota=$usquota");
  }
  if(defined($uhquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxHardQuota=$uhquota");
  }
  if(defined($snd_skel_Uid)){
   push(@u_attr,"--attribute");
   push(@u_attr,"cipuxSkeletonUid=$snd_skel_Uid");
  }

    #system("/usr/bin/cipux_add -u -w '$password' $u_attr");

    # exec commad with pipe, give string, ref to array
    &task_exec_cmd_writing($password,\@u_attr);

    # normal skeleton
    if($grole ne "none"){
      my $cmd = "/usr/bin/cipux_refine_homedir restore '/skole/tjener/home0' $grole $id";
      &debug($SCRIPT,'* normal skeleton: $cmd');
      system($cmd);
    }

    # Forbach skeleton
    if(defined($snd_skel_Uid)){
      my $cmd = "/usr/bin/cipux_refine_homedir restore '/skole/tjener/home0' $snd_skel_Uid $id";
      &debug($SCRIPT,'* forbach skeleton: $cmd');
      system($cmd);
    }

    # add to user to role group
    system("$set -a -c '$grole' -e memberUid -v '$id'");
    #system("$set -a -c '$role' -e member -v 'uid=$id,$dn'");#QUESTION: not more in sage debian-edu?

    # add user to default class
    if(defined($default_CID)){
      # add to default group (for class, or exam)
      system("$set -a -c '$default_CID' -e memberUid -v '$id'");
      #system("$set -a -c '$default_CID' -e member -v 'uid=$id,$dn'");#QUESTION: not more in sage debian-edu?
    }

  }else{
    print "ERROR: can't create role $grole!\n" if $mode eq 'console' ;
  }

}



# +------------------------------------------------------------------------+
# | cipux_task_create_examination_user                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_examination_user{
  # &cipux_task_create_examination_user('cgi',$password,$login,$hquota,$squota,$role,$firstname,$lastname);
  my $mode       = shift; # cgi
  my $id         = shift; # grammar   group: cn, gid, course_login; user: uid, login
  my $password   = shift; # secret
  my $uhquota    = shift; # 1000
  my $usquota    = shift; # 800
  #  my $group_type = shift; # private
  my $role       = shift; # student
  my $firstname  = shift; # cipux
  my $lastname   = shift; # course
  my $group      = shift;# chemie   memberUid
  my $leader     = shift;
  my $course     = shift;
  my $ghquota    = shift; # 2000
  my $gsquota    = shift; # 1800

  # '$password' '$login' '$firstname '$lastname' '$role' '$uhquota' '$usquota' '$ghquota' 'gsquota' '$group'
  #system("/usr/bin/cipux_add -c -w '$password' --attribute cn='$course' --attribute groupType='private' --attribute cipuxSoftQuota='$usquota' --attribute cipuxHardQuota='$uhquota' --attribute cipuxGroupLeaderUid='$leader' --attribute cipuxRole='examination' --attribute cipuxInternetStatus=accept");

  $password= &test_password_existence($password);
  my @c_attr = ($add, "-c", "-W", "--attribute", "cn=$course",
                                    "--attribute", "groupType=private",
                                    "--attribute", "cipuxRole=examination",
                                    "--attribute", "cipuxInternetStatus=accept",
                                    "--attribute", "cipuxGroupLeaderUid=$leader",
                                    "--attribute", "cipuxRemark=created by CipUX::Task ($mode)");

  if(defined($gsquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxSoftQuota=$gsquota");
  }
  if(defined($ghquota)){
    push(@c_attr,"--attribute");
    push(@c_attr,"cipuxHardQuota=$ghquota");
  }

  #system("/usr/bin/cipux_add -c -w '$password' $c_attr");
  # exec commad with pipe, give string, ref to array
  &task_exec_cmd_writing($password,\@c_attr);

  # create login
  #system("/usr/bin/cipux_add -u -w '$password'
  #--attribute cipuxHardQuota='$uhquota'
  #--attribute cipuxSoftQuota='$usquota'
  #--attribute cipuxFirstname='cipux'
  #--attribute cipuxLastname='examination'
  #--attribute cipuxRole='course'
  #--attribute cipuxInternetStatus=accept
  #--attribute uid='$course' ");

  # TODO QUESTION: Is this right? cipuxRole=course
  my @u_attr = ($add, "-u", "-W",
                     "--attribute", "uid=$course",
                     "--attribute", "cipuxRole=course",
                     "--attribute", "cipuxInternetStatus=accept",
                     "--attribute", "cipuxRemark=created by CipUX::Task ($mode)",
                     "--attribute", "cipuxFirstname=CipUX",
                     "--attribute", "cipuxLastname=examination");

  if(defined($usquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxSoftQuota=$usquota");
  }
  if(defined($uhquota)){
    push(@u_attr,"--attribute");
    push(@u_attr,"cipuxHardQuota=$uhquota");
  }

  # exec commad with pipe, give string, ref to array
  &task_exec_cmd_writing($password,\@u_attr);
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_course                                               |
# +------------------------------------------------------------------------+
sub cipux_task_delete_course{
  my $mode   = shift;
  my $cid    = shift;
  &cipux_task_delete_user($mode,$cid);
  #print "DELETE";
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_skel                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_delete_skel{
  my $mode   = shift;
  my $cid    = shift;
  &cipux_task_delete_user($mode,$cid);
  print "DELETE";
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_role                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_delete_role{
  my $mode   = shift;
  my $cid    = shift;
  &cipux_task_delete_user($mode,$cid);
  print "DELETE";
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_user                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_delete_user{
  my $mode = shift;
  my $uid  = shift;
  &cipux_task_backup_user_homedir($mode,$uid);
  &cipux_task_delete_user_homedir($mode,$uid);
  system("/usr/bin/cipux_del -c '$uid'");
  system("/usr/bin/cipux_del -u '$uid'");
}

# +------------------------------------------------------------------------+
# | cipux_task_enable_user_posix_account                                   |
# +------------------------------------------------------------------------+
sub cipux_task_enable_user_posix_account{
  my $mode = shift;
  my $uid  = shift;
  #
  my $attr = 'userPassword';
  my $u_pwd = `cipux_get_value -u '$uid' -e $attr`;
  my $c_pwd = `cipux_get_value -c '$uid' -e $attr`;
  chomp($u_pwd);
  chomp($c_pwd);
  $u_pwd =~ s/^\[DISABLED\]//;
  $c_pwd =~ s/^\[DISABLED\]//;

  #system("$set -u $uid -e $attr -v '$u_pwd'");
  #system("$set -c $uid -e $attr -v '$c_pwd'");
  my @cmd=($set,"-u", $uid,"-e",$attr,"-V");
  &task_exec_cmd_writing($u_pwd,\@cmd);
  @cmd=($set,"-c", $uid,"-e",$attr,"-V");
  &task_exec_cmd_writing($c_pwd,\@cmd);
  # expire system
  system("$set -u $uid -e cipuxStatus -v 'idle'");
}
# +------------------------------------------------------------------------+
# | cipux_task_enable_user_samba_account                                   |
# +------------------------------------------------------------------------+
sub cipux_task_enable_user_samba_account{
  my $mode = shift;
  my $uid  = shift;
  #
  my $out = `cipux_get_value -u $uid -e sambaAcctFlags`;
  chomp($out);
  if($out eq '[D          ]'){
     system("cipux_set_value -u $uid -e sambaAcctFlags -v '[U          ]'");
  }
  # expire system
  system("$set -u $uid -e cipuxStatus -v 'idle'")
}
# +------------------------------------------------------------------------+
# | cipux_task_enable_course_posix_account                                 |
# +------------------------------------------------------------------------+
sub cipux_task_enable_course_posix_account{
  my $mode = shift;
  my $cid  = shift;
  #
  &cipux_task_enable_user_posix_account($mode,$cid);
}
# +------------------------------------------------------------------------+
# | cipux_task_enable_course_samba_account                                 |
# +------------------------------------------------------------------------+
sub cipux_task_enable_course_samba_account{
  my $mode = shift;
  my $cid  = shift;
  #
  &cipux_task_enable_user_samba_account($mode,$cid);
}
# +------------------------------------------------------------------------+
# | cipux_task_disable_user_posix_account                                  |
# +------------------------------------------------------------------------+
sub cipux_task_disable_user_posix_account{
  my $mode = shift;
  my $uid  = shift;
  #
  my $attr = 'userPassword';
  my $u_pwd = `cipux_get_value -u '$uid' -e $attr`;
  my $c_pwd = `cipux_get_value -c '$uid' -e $attr`;
  chomp($u_pwd);
  chomp($c_pwd);
  $u_pwd =~ s/\[DISABLED\]+//g;
  $c_pwd =~ s/\[DISABLED\]+//g;
  #system("$set -u $uid -e $attr -v '[DISABLED]$u_pwd'");
  #system("$set -c $uid -e $attr -v '[DISABLED]$c_pwd'");
  my @cmd=($set,"-u", $uid,"-e",$attr,"-V");
  &task_exec_cmd_writing("[DISABLED]$u_pwd",\@cmd);
  @cmd=($set,"-c", $uid,"-e",$attr,"-V");
  &task_exec_cmd_writing("[DISABLED]$c_pwd",\@cmd);
  # expire system
  system("$set -u $uid -e cipuxStatus -v 'disabled'")

}
# +------------------------------------------------------------------------+
# | cipux_task_disable_user_samba_account                                  |
# +------------------------------------------------------------------------+
sub cipux_task_disable_user_samba_account{
  my $mode = shift;
  my $uid  = shift;
  #
  my $out = `cipux_get_value -u $uid -e sambaAcctFlags`;
  chomp($out);
  if($out eq '[U          ]'){
     system("cipux_set_value -u $uid -e sambaAcctFlags -v '[D          ]'");
  }
  # expire system
  system("$set -u $uid -e cipuxStatus -v 'disabled'")
}
# +------------------------------------------------------------------------+
# | cipux_task_disable_course_posix_account                                |
# +------------------------------------------------------------------------+
sub cipux_task_disable_course_posix_account{
  my $mode = shift;
  my $cid  = shift;
  #
  &cipux_task_disable_user_posix_account($mode,$cid);
}
# +------------------------------------------------------------------------+
# | cipux_task_disable_course_samba_account                                |
# +------------------------------------------------------------------------+
sub cipux_task_disable_course_samba_account{
  my $mode = shift;
  my $cid  = shift;
  #
  &cipux_task_disable_user_samba_account($mode,$cid);
}
# +------------------------------------------------------------------------+
# | cipux_task_enable_user                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_enable_user{
  my $mode = shift;
  my $uid  = shift;
  &cipux_task_enable_user_posix_account($mode,$uid);
  &cipux_task_enable_user_samba_account($mode,$uid);
}

# +------------------------------------------------------------------------+
# | cipux_task_enable_course                                               |
# +------------------------------------------------------------------------+
sub cipux_task_enable_course{
  my $mode = shift;
  my $cid  = shift;
  &cipux_task_enable_user($mode,$cid);
}

# +------------------------------------------------------------------------+
# | cipux_task_disable_user                                                |
# +------------------------------------------------------------------------+
sub cipux_task_disable_user{
  my $mode = shift;
  my $uid  = shift;
  #
  &cipux_task_disable_user_posix_account($mode,$uid);
  &cipux_task_disable_user_samba_account($mode,$uid);
}

# +------------------------------------------------------------------------+
# | cipux_task_disable_course                                               |
# +------------------------------------------------------------------------+
sub cipux_task_disable_course{
  my $mode = shift;
  my $cid  = shift;
  &cipux_task_disable_user($mode,$cid);
}


# +------------------------------------------------------------------------+
# | cipux_task_list_courses                                                |
# +------------------------------------------------------------------------+
sub cipux_task_list_courses{
  my $mode  = shift;

  my @role = `$get -U -e cipuxRole`;
  my @return =();
  foreach (@role){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $r eq 'share';
  }

  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_create_room                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_create_room{
  my $mode  = shift;
  # mandatory
  my $cn                  = shift;
  my $cipuxNetworkAddress = shift;
  my $cipuxNetworkMask    = shift;
  my $cipuxGatewayAddress = shift;
  my $cipuxGatewayMask    = shift;
  # optional
  my $description         = shift;
  # future
 # my $cipuxFaculty        = shift;
 # my $cipuxAddress        = shift;
 # my $cipuxImageSlotNumber= shift; #integer
 # my $roomNumber          = shift;
  #my $telephoneNumber     = shift;

# cipux_add -r
# --attribute cn=roomz4
# --attribute 'cipuxNetworkAddress=192.168.0.0'
# --attribute 'cipuxNetworkMask=255.255.255.0'
# --attribute 'cipuxGatewayAddress=129.70.108.1'
# --attribute 'cipuxGatewayMask=255.255.255.0'

  my $r_attr =  " --attribute cn='$cn'";
  $r_attr .= " --attribute cipuxNetworkAddress='$cipuxNetworkAddress'";
  $r_attr .= " --attribute cipuxNetworkMask='$cipuxNetworkMask'";
  $r_attr .= " --attribute cipuxGatewayAddress='$cipuxGatewayAddress'";
  $r_attr .= " --attribute cipuxGatewayMask='$cipuxGatewayMask'";
  $r_attr .= " --attribute description='$description'" if defined($description);
  system("/usr/bin/cipux_add -r $r_attr");

}
# +------------------------------------------------------------------------+
# | cipux_task_delete_room                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_delete_room{
  my $mode  = shift;
  my $cn    = shift;
  system("/usr/bin/cipux_del -r '$cn'");
}

# +------------------------------------------------------------------------+
# | cipux_task_create_machine                                              |
# +------------------------------------------------------------------------+
sub cipux_task_create_machine{
  my $mode  = shift;
  # mandatory
  # cipux_add -m --attribute uid=win59$
  # --attribute cipuxMacAddress=00:00:00:00:00
  # --attribute cipuxRoomName=cippool
  # --attribute cipuxClientNumber=1
  # --attribute cipuxImageSlotNumber=1
  # --attribute cipuxHardwareTypeNumber=1
  my $mid                      = shift;
  my $cipuxClientNumber        = shift || 0; # 0 if not undef or empty
  my $cipuxMacAddress          = shift || "00:00:00:00:00";
  my $cipuxRoomName            = shift || "default";
  my $cipuxMaster              = shift || "FALSE";
  my $cipuxImageSlotNumber     = shift || 1;
  my $cipuxHardwareTypeNumber  = shift || 1;


  $mid =~ s/\$$//g; # delete trailing $  (Samba server will use this to add a machine)
                    # the cipux_add script will add a $ later 
		    
  my $m_attr =  " --attribute uid='$mid'";
  $m_attr .= " --attribute cipuxClientNumber='$cipuxClientNumber'";
  $m_attr .= " --attribute cipuxMacAddress='$cipuxMacAddress'";
  $m_attr .= " --attribute cipuxRoomName='$cipuxRoomName'";
  $m_attr .= " --attribute cipuxMaster='$cipuxMaster'";
  $m_attr .= " --attribute cipuxImageSlotNumber='$cipuxImageSlotNumber'";
  $m_attr .= " --attribute cipuxHardwareTypeNumber='$cipuxHardwareTypeNumber'";

  system("/usr/bin/cipux_add -m $m_attr");

}
# +------------------------------------------------------------------------+
# | cipux_task_delete_machine                                              |
# +------------------------------------------------------------------------+
sub cipux_task_delete_machine{
  my $mode  = shift;
  my $cn    = shift;
  system("/usr/bin/cipux_del -m '$cn'");
}

# +------------------------------------------------------------------------+
# | cipux_task_create_hardware_type                                        |
# +------------------------------------------------------------------------+
sub cipux_task_create_hardware_type{
  my $mode  = shift;
  # NAME 'cipuxHardwareType'
  # DESC 'CipUX Auxiliary Hardware Type HAL 3.0'
  # MUST ( cn $
  #             cipuxHardwareTypeName $
  #             cipuxHardwareTypeProtect $
  #             cipuxHardwareTypeNic )
  # mandatory
  my $cn                       = shift;
  my $cipuxHardwareTypeName    = shift;
  my $cipuxHardwareTypeProtect = shift;
  my $cipuxHardwareTypeNic     = shift;

  my $t_attr = " --attribute cn='$cn'";
  $t_attr  .= "  --attribute cipuxHardwareTypeName='$cipuxHardwareTypeName'";
  $t_attr  .= "  --attribute cipuxHardwareTypeProtect='$cipuxHardwareTypeProtect'";
  $t_attr  .= "  --attribute cipuxHardwareTypeNic='$cipuxHardwareTypeNic'";
  system("/usr/bin/cipux_add -t $t_attr");
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_hardware_type                                        |
# +------------------------------------------------------------------------+
sub cipux_task_delete_hardware_type{
  my $mode  = shift;
  my $cn    = shift;
  system("/usr/bin/cipux_del -t '$cn'");
}
# +------------------------------------------------------------------------+
# | cipux_task_create_image_slot                                           |
# +------------------------------------------------------------------------+
sub cipux_task_create_image_slot{
  my $mode  = shift;
  #      NAME 'cipuxImageSlot'
  #      DESC 'CipUX Auxiliary Image Slot HAL 3.0'
  #      MUST ( cn $
  #             cipuxImageSlotName $
  #             cipuxImageSlotProtect $
  #             cipuxImageSlotEnabled )
  # mandatory
  my $cn                    = shift;
  my $cipuxImageSlotName    = shift;
  my $cipuxImageSlotProtect = shift;
  my $cipuxImageSlotEnabled = shift;

  my $s_attr = " --attribute cn='$cn'";
  $s_attr    .= " --attribute cipuxImageSlotName='$cipuxImageSlotName'";
  $s_attr    .= " --attribute cipuxImageSlotProtect='$cipuxImageSlotProtect'";
  $s_attr    .= " --attribute cipuxImageSlotEnabled='$cipuxImageSlotEnabled'";

  system("/usr/bin/cipux_add -s $s_attr");
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_image_slot                                           |
# +------------------------------------------------------------------------+
sub cipux_task_delete_image_slot{
  my $mode  = shift;
  my $cn    = shift;
  system("/usr/bin/cipux_del -s '$cn'");
}

# +------------------------------------------------------------------------+
# | cipux_task_create_image                                                |
# +------------------------------------------------------------------------+
sub cipux_task_create_image{
  my $mode  = shift;
  #      NAME 'cipuxImage'
  #      DESC 'CipUX Auxiliary Image HAL 3.0'
  #      MUST ( cn $
  #             cipuxImageName $
  #             cipuxImageType $
  #             cipuxImageProtect $
  #             cipuxImageEnabled $
  #             cipuxImageSize $
  #             cipuxChecksum $
  #             cipuxImageSlotNumber $
  # mandatory
  my $cn                      = shift;
  my $cipuxImageName          = shift;
  my $cipuxImageType          = shift;
  my $cipuxImageProtect       = shift;
  my $cipuxImageEnabled       = shift;
  my $cipuxImageSize          = shift;
  my $cipuxChecksum           = shift;
  my $cipuxHardwareTypeNumber = shift;
  my $cipuxImageSlotNumber    = shift;

  my $i_attr = " --attribute cn='$cn'";
  $i_attr   .= "  --attribute cipuxImageName='$cipuxImageName'";
  $i_attr   .= "  --attribute cipuxImageType='$cipuxImageType'";
  $i_attr   .= "  --attribute cipuxImageProtect='$cipuxImageProtect'";
  $i_attr   .= "  --attribute cipuxImageEnabled='$cipuxImageEnabled'";
  $i_attr   .= "  --attribute cipuxImageSize='$cipuxImageSize'";
  $i_attr   .= "  --attribute cipuxChecksum='$cipuxChecksum'";
  $i_attr   .= "  --attribute cipuxImageSlotNumber='$cipuxImageSlotNumber'";
  $i_attr   .= "  --attribute cipuxHardwareTypeNumber='$cipuxHardwareTypeNumber'";
  system("/usr/bin/cipux_add -i $i_attr");
}

# +------------------------------------------------------------------------+
# | cipux_task_delete_image                                                |
# +------------------------------------------------------------------------+
sub cipux_task_delete_image{
  my $mode  = shift;
  my $cn    = shift;
  system("/usr/bin/cipux_del -i '$cn'");
}

# +------------------------------------------------------------------------+
# | cipux_task_list_user_node_role                                         |
# +------------------------------------------------------------------------+
sub cipux_task_list_user_node_role{
  my $mode  = shift;
  # mandatory
  my $role  = shift;# share, account

  my @role = `$get -U -e cipuxRole`;
  my @return =();
  foreach (@role){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $r eq $role;
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_roles                                                  |
# +------------------------------------------------------------------------+

sub cipux_task_list_roles{
  my $mode  = shift;
  return cipux_task_list_user_node_role('task','role');

}

# +------------------------------------------------------------------------+
# | cipux_task_list_skels                                                  |
# +------------------------------------------------------------------------+

sub cipux_task_list_skels{
  my $mode  = shift;
  return cipux_task_list_group_node_role('task','skel');
}


# +------------------------------------------------------------------------+
# | cipux_task_list_group_node_role                                        |
# +------------------------------------------------------------------------+
sub cipux_task_list_group_node_role{
  my $mode  = shift;
  # mandatory
  my $role  = shift;# student, teacher

  my @role = `$get -C -e cipuxRole`;
  my @return =();
  foreach (@role){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $r eq $role;
  }
  return @return;
}


# +------------------------------------------------------------------------+
# | cipux_task_list_teachers                                               |
# +------------------------------------------------------------------------+
sub cipux_task_list_teachers{
  my $mode  = shift;
  cipux_task_list_group_node_role('task','teacher');
}

# +------------------------------------------------------------------------+
# | cipux_task_list_students                                               |
# +------------------------------------------------------------------------+
sub cipux_task_list_students{
  my $mode  = shift;
  cipux_task_list_group_node_role('task','student');
}

# +------------------------------------------------------------------------+
# | cipux_task_list_guests                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_list_guests{
  my $mode  = shift;
  cipux_task_list_group_node_role('task','guest');
}

# +------------------------------------------------------------------------+
# | cipux_task_list_rooms                                                  |
# +------------------------------------------------------------------------+
sub cipux_task_list_rooms{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @rooms  = `$get -R -e cn`;
  my @return =();
  foreach (@rooms){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u);
  }

  #my @return = &split_to_first_column(\@out);
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_machines                                               |
# +------------------------------------------------------------------------+
sub cipux_task_list_machines{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @machines  = `$get -M -e uid`;
  my @return =();
  foreach (@machines){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u);
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_hardware_types                                         |
# +------------------------------------------------------------------------+
sub cipux_task_list_hardware_types{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @types  = `$get -T -e uid`;
  my @return =();
  foreach (@types){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u);
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_image_slots                                            |
# +------------------------------------------------------------------------+
sub cipux_task_list_image_slots{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @slots  = `$get -S -e uid`;
  my @return =();
  foreach (@slots){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u);
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_images                                                 |
# +------------------------------------------------------------------------+
sub cipux_task_list_images{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @images  = `$get -I -e uid`;
  my @return =();
  foreach (@images){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u);
  }
  return @return;
}




# +------------------------------------------------------------------------+
# |   cipux_task_list_user_members_of_course                               |
# +------------------------------------------------------------------------+
sub cipux_task_list_user_members_of_course{
  my $mode = shift;
  my $cid  = shift;

  my @out = `$get -c '$cid' -e memberUid`;
  my @list = &split_to_first_column(\@out);
  my @return = ();
  foreach(@list){

    # only REAL member are interesting
    push(@return,$_) if $_ ne $cid;
  }
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_objects                                                  |
# +------------------------------------------------------------------------+
sub cipux_task_list_objects{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @uid  = `$get -U -e uid`;
  my @return =();
  foreach (@uid){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $u ne 'root';
  }

  #my @return = &split_to_first_column(\@out);
  return @return;
}

# +------------------------------------------------------------------------+
# | cipux_task_list_users                                                  |
# +------------------------------------------------------------------------+
sub cipux_task_list_users{
  my $mode  = shift; # cgi

  #my @out    = `$get -U -e 'uid'`;
  my @role  = `$get -C -e groupType`;
  my @return =();
  foreach (@role){
    chomp;
    my($u,$r) = split(/\t/);
    push(@return,$u) if $r eq 'private';
  }

  #my @return = &split_to_first_column(\@out);
  return @return;
}


# +------------------------------------------------------------------------+
# | cipux_task_list_rpc_functions                                          |
# +------------------------------------------------------------------------+
sub cipux_task_list_rpc_functions     {
#=> '0,0:;',
 my @return;
  foreach(@EXPORT){
    if(/cipux_task/){
      s/\&//g;
      push(@return, $_);
    }
  }
  return @return;
}


# +------------------------------------------------------------------------+
# | cipux_task_remove_nonexistent_users_from_all_courses                   |
# +------------------------------------------------------------------------+
sub cipux_task_remove_nonexistent_users_from_all_courses{
  my $mode = shift;

  system("/usr/bin/cipux_bot_extinct_from_courses -a");
}

# +------------------------------------------------------------------------+
# | cipux_task_remove_user_from_course                                     |
# +------------------------------------------------------------------------+
sub cipux_task_remove_user_from_course{
  my $mode       = shift;
  my $cid        = shift;
  my $member_uid = shift;

  system("$set -d -c '$cid' -e 'memberUid' -v '$member_uid'");
}

# +------------------------------------------------------------------------+
# | cipux_task_remove_user_from_all_courses                                     |
# +------------------------------------------------------------------------+
sub cipux_task_remove_user_from_all_courses{
  my $mode = shift;
  my $uid  = shift;
  system("/usr/bin/cipux_extinct_from_courses -u '$uid'");
}


# +------------------------------------------------------------------------+
# | cipux_task_enable_internet_for_course                                  |
# +------------------------------------------------------------------------+
sub cipux_task_enable_internet_for_course{
  my $mode   = shift;
  my $course  = shift;
  system("/usr/bin/cipux_internet_access -e -c '$course'");
}

# +------------------------------------------------------------------------+
# | cipux_task_disable_internet_for_course                                  |
# +------------------------------------------------------------------------+
sub cipux_task_disable_internet_for_course{
  my $mode   = shift;
  my $course  = shift;
  system("/usr/bin/cipux_internet_access -d -c '$course'");
}

# +------------------------------------------------------------------------+
# | cipux_task_start_exam                                  |
# +------------------------------------------------------------------------+
sub cipux_task_start_exam{
  my $mode    = shift;
  my $course  = shift;
  my $teacher = shift;

  system("/usr/bin/cipux_start_exam $course $teacher ");
}

# +------------------------------------------------------------------------+
# | cipux_task_collect_exam                                  |
# +------------------------------------------------------------------------+
sub cipux_task_collect_exam{
  my $mode    = shift;
  my $course  = shift;
  my $teacher = shift;
  system("/usr/bin/cipux_collect_exam $course $teacher ");
}

# +------------------------------------------------------------------------+
# | cipux_task_end_exam                                  |
# +------------------------------------------------------------------------+
sub cipux_task_end_exam{
  my $mode    = shift;
  my $course  = shift;
  my $teacher = shift;
  system("/usr/bin/cipux_end_exam $course $teacher ");
}

# +------------------------------------------------------------------------+
# | cipux_task_change_cipux_config_variable_value                          |
# +------------------------------------------------------------------------+
sub cipux_task_change_cipux_config_variable_value {
  my $mode    = shift;
  my $system  = shift; # cipux, cibot
  my $value   = shift; # $in{cipux_variable}=$in{cipux_variable_value}
  system("$set -a -x '$system' -e cipuxVariable -v '$value'");
}

# +------------------------------------------------------------------------+
# | cipux_task_tell_me_user_firstname                                       |
# +------------------------------------------------------------------------+
sub cipux_task_tell_me_user_firstname{
  my $mode    = shift;
  my $uid     = shift; # login
  print "[$get -u $uid -e 'cipuxFirstname']";
  my @out    = `$get -u $uid -e 'cipuxFirstname'`;
  foreach(@out){
    chomp;
  }
  return @out;
}
      
# +------------------------------------------------------------------------+
# | cipux_task_tell_me_user_lastname                                       |
# +------------------------------------------------------------------------+
sub cipux_task_tell_me_user_lastname{
  my $mode    = shift;
  my $uid     = shift; # login
  my $out    = `$get -u $uid -e 'cipuxLastname'`;
  chomp($out);
  return $out;
}

# +------------------------------------------------------------------------+
# | cipux_task_tell_me_user_canonical_name                                 |
# +------------------------------------------------------------------------+
sub cipux_task_tell_me_user_canonical_name{
  my $mode    = shift;
  my $uid     = shift; # login
  my $out    = `$get -u $uid -e 'cn'`;
  chomp($out);
  return $out;
}
# +------------------------------------------------------------------------+
# | cipux_task_tell_me_user_firstname                                       |
# +------------------------------------------------------------------------+
sub cipux_task_tell_me_user_role{
  my $mode    = shift;
  my $uid     = shift; # login
  # the Role is in the group node!
  my $out    = `$get -c $uid -e 'cipuxRole'`;
  chomp($out);
  return $out;
}


1;

__END__

#===========================================================================
#==== Documentation ========================================================
#===========================================================================

=head1 SEE ALSO

See the CipUX webpage and the  manual at L<http://www.cipux.org>
See the mailing list L<http://www.cipux.org/mailman/listinfo/>

=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 1999-2005 by Christian Kuelker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=cut
